/* Keep these headers */
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <tice.h>

/* Standard headers */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Shared libraries */
#include <lib/ce/graphx.h>
#include <lib/ce/fileioc.h>
#include <lib/ce/keypadc.h>
#include "gfx/player_gfx.h"
#include "gfx/tile_gfx.h"

/* Tilemap defines */
#define TILE_WIDTH	11
#define TILE_HEIGHT	11
#define TILEMAP_WIDTH	30
#define TILEMAP_HEIGHT	18
#define TILEMAP_X_OFFSET -5
#define TILEMAP_Y_OFFSET -5

/* Palette defines */
#define SPIKE_COLOR	107
#define BLOCK_COLOR	227
#define WHITE		255

extern uint8_t TITOL_map[];

const gfx_image_t *tile_ptrs[] = {
   block, block, up_spike, down_spike, left_spike, right_spike, start_pipe, end_pipe, door, door, button
};

/* Put function prototypes here */
void draw_tile( uint8_t tile_x, uint8_t tile_y );
void draw_tilemap(void);

/* Put all your code here */
void main(void) {
	int player_x, player_y;
	uint8_t block_color[3], spike_color[3];
	uint8_t key7;
	gfx_image_t *player_back;
	
	player_x = player_y = 0;
	
	/* Seed the random number generator */
	srand(rtc_Time());
	
	/* Allocate space for the temporary sprite to save the background */
	malloc(0);
	player_back = gfx_AllocSprite( player->width, player->height, malloc );
	
	/* Initialize graphics */
	gfx_Begin(gfx_8bpp);
	
	/* Set up the palette */
	gfx_SetPalette(tile_gfx_pal, sizeof(tile_gfx_pal), 0);
	gfx_SetDrawBuffer();
	
	gfx_palette[BLOCK_COLOR] = randInt( 0x100, 0xF000 );
	gfx_palette[SPIKE_COLOR] = randInt( 0x100, 0xF000 );
	
	/* Program code*/
	draw_tilemap();
	
	gfx_GetSprite( player_back, player_x, player_y );
	
	while( kb_ScanGroup(kb_group_6) != kb_Clear ) {
		if( (key7 = kb_ScanGroup(kb_group_7))) {
			
			gfx_Sprite_NoClip( player_back, player_x, player_y );
			
			if ( key7 & kb_Up ) {
				player_y--;
			}
			if ( key7 & kb_Down ) {
				player_y++;
			}
			if ( key7 & kb_Left ) {
				player_x--;
			}
			if ( key7 & kb_Right ) {
				player_x++;
			}
			
			gfx_GetSprite_NoClip( player_back, player_x, player_y );
		}
		
		gfx_TransparentSprite_NoClip( player, player_x, player_y );
		gfx_Blit( gfx_buffer );
	}
	
	/* Clean up and exit */
	gfx_End(); prgm_CleanUp();
}

void draw_tilemap(void) {
	uint8_t tilemap_x, tilemap_y;
	
	gfx_FillScreen( WHITE );
	for( tilemap_x = 0; tilemap_x < TILEMAP_WIDTH; tilemap_x++ ) {
		for( tilemap_y = 0; tilemap_y < TILEMAP_HEIGHT; tilemap_y++ ) {
			draw_tile( tilemap_x, tilemap_y );
		}
	}
}

void draw_tile( uint8_t tile_x, uint8_t tile_y ) {
	unsigned xpos = TILE_WIDTH * tile_x;
	uint8_t ypos = TILE_HEIGHT * tile_y;
	uint8_t tile = TITOL_map[TILEMAP_WIDTH * tile_y + tile_x];
	
	switch( tile ) {
		case 0:
			break;
		case 1:
			gfx_Sprite( block, xpos, ypos );
			break;
		case 8:
		case 9:
			gfx_TransparentSprite_NoClip( door, xpos, ypos );
			break;
		default:
			gfx_TransparentSprite_NoClip( tile_ptrs[tile], xpos, ypos );
			break;
	}
}